import psycopg2

from ovretl.db_utils.fetch_db_credentials import fetch_db_credentials


def execute_simple_update(
    set_column: str, set_value, where_column: str, where_value, table_name: str, dbname: str, stage="dev",
):
    credentials = fetch_db_credentials(stage)
    conn = psycopg2.connect(
        host=credentials["host"],
        user=credentials["username"],
        password=credentials["password"],
        dbname=dbname,
        port=6543,
    )
    cur = conn.cursor()
    query = """UPDATE "{0}" SET "{1}" = %s WHERE "{2}" = CAST(%s as character varying [])""".format(
        table_name, set_column, where_column
    )
    cur.execute(query, (set_value, where_value))
    print(cur.query)
    print("Updated {} rows".format(cur.rowcount))
    cur.close()
    conn.commit()
    conn.close()



from datetime import datetime, timedelta

import numpy as np
import pandas as pd
from ovretl.tracking_utils.check_shipment_in_progress import check_shipment_in_progress

empty_events_df = pd.DataFrame(data={"shipment_id": []})


def test_check_shipment_in_progress():
    quotation = pd.Series(data=["0", "to_be_requoted", False], index=["shipment_id", "shipment_status", "cancelled"],)
    assert check_shipment_in_progress(quotation, empty_events_df) == "to_be_requoted"
    finished = pd.Series(data=["0", "booked", False], index=["shipment_id", "shipment_status", "cancelled"],)
    assert check_shipment_in_progress(finished, empty_events_df) == "booked"

    is_arrived = pd.Series(
        data=["0", "booked", np.nan, np.nan, datetime.now() - timedelta(days=1), np.nan, np.nan, False,],
        index=[
            "shipment_id",
            "shipment_status",
            "finished_date",
            "pickup_date",
            "departure_date",
            "arrival_date",
            "delivery_date",
            "cancelled",
        ],
    )
    assert (
        check_shipment_in_progress(
            is_arrived, pd.DataFrame(data={"shipment_id": ["0", "0"], "date_type": ["actual", "actual"]}),
        )
        == "booked"
    )
    is_not_arrived_and_active = pd.Series(
        data=["0", "booked", np.nan, np.nan, datetime.now() - timedelta(days=1), np.nan, np.nan, False,],
        index=[
            "shipment_id",
            "shipment_status",
            "finished_date",
            "pickup_date",
            "departure_date",
            "arrival_date",
            "delivery_date",
            "cancelled",
        ],
    )
    assert (
        check_shipment_in_progress(
            is_not_arrived_and_active,
            pd.DataFrame(data={"shipment_id": ["0", "0"], "date_type": ["actual", "estimated"]}),
        )
        == "in_progress"
    )



from typing import List

import psycopg2

from ovretl.db_utils.fetch_db_credentials import fetch_db_credentials


def execute_complexe_update_three_cols(
    set_column: str,
    set_data: List,
    where_column_1: str,
    where_data_1: List,
    where_column_2: str,
    where_data_2: List,
    table_name: str,
    dbname: str,
    stage="dev",
):
    """
    Update table in DB with 3 vectors, one to set, 2 to know where.
    :param set_column: Name of the column to update
    :param set_data: New data
    :param where_column: Name of the column to lookup
    :param where_data: Data to lookup
    :param table_name: table name
    :param dbname: database name
    :param stage: dev or prod
    :return: void
    """
    assert len(set_data) == len(where_data_1) == len(where_data_2)
    values_to_insert = list(zip(set_data, where_data_1, where_data_2))
    credentials = fetch_db_credentials(stage)
    conn = psycopg2.connect(
        host=credentials["host"],
        user=credentials["username"],
        password=credentials["password"],
        dbname=dbname,
        port=6543,
    )
    cur = conn.cursor()
    from psycopg2.extras import execute_values

    execute_values(
        cur,
        """UPDATE "{0}" SET "{1}" = data."{1}" FROM (VALUES %s) AS data ({1}, {2}, {3}) WHERE "{0}".{2} = data.{2} AND "{0}".{3} = data.{3} """.format(
            table_name, set_column, where_column_1, where_column_2
        ),
        values_to_insert,
    )

    print("Updated {} rows".format(cur.rowcount))
    cur.close()
    conn.commit()
    conn.close()



import pandas as pd


def shipments_with_total_transit_time(shipments_with_employees_df: pd.DataFrame) -> pd.DataFrame:
    active_shipments_df = shipments_with_employees_df[
        (shipments_with_employees_df["shipment_status"] != "finished") & (~shipments_with_employees_df["cancelled"])
    ]
    active_shipments_df.loc[
        :, ["transit_time_door_to_port", "transit_time_port_to_door", "transit_time"]
    ] = active_shipments_df[["transit_time_door_to_port", "transit_time_port_to_door", "transit_time"]].fillna(0)
    active_shipments_df.loc[:, "total_transit_time"] = (
        active_shipments_df["transit_time_door_to_port"]
        + active_shipments_df["transit_time"]
        + active_shipments_df["transit_time_port_to_door"]
    )
    return active_shipments_df



from typing import List

import psycopg2

from ovretl.db_utils.fetch_db_credentials import fetch_db_credentials


def execute_complexe_update(
    set_column: str, set_data: List, where_column: str, where_data: List, table_name: str, dbname: str, stage="dev",
):
    """
    Update table in DB with 2 vectors, one to set, one to know where.
    :param set_column: Name of the column to update
    :param set_data: New data
    :param where_column: Name of the column to lookup
    :param where_data: Data to lookup
    :param table_name: table name
    :param dbname: database name
    :param stage: dev or prod
    :return: void
    """
    assert len(set_data) == len(where_data)
    values_to_insert = list(zip(set_data, where_data))
    credentials = fetch_db_credentials(stage)
    conn = psycopg2.connect(
        host=credentials["host"],
        user=credentials["username"],
        password=credentials["password"],
        dbname=dbname,
        port=6543,
    )
    cur = conn.cursor()
    from psycopg2.extras import execute_values

    if set_column != where_column:
        execute_values(
            cur,
            """UPDATE "{0}" SET "{1}" = data."{1}" FROM (VALUES %s) AS data ({1}, {2}) WHERE "{0}".{2} = data.{2}""".format(
                table_name, set_column, where_column
            ),
            values_to_insert,
        )
    else:
        execute_values(
            cur,
            """UPDATE "{0}" SET "{1}" = data."{1}" FROM (VALUES %s) AS data ({1}, {1}_old) WHERE "{0}".{1} = data.{1}_old""".format(
                table_name, set_column
            ),
            values_to_insert,
        )
    print("Updated {} rows".format(cur.rowcount))
    cur.close()
    conn.commit()
    conn.close()



import pandas as pd
from datetime import date, timedelta
import numpy as np
from typing import NamedTuple
from ovretl.shipment_orchestration_utils.utils import string_date_to_datetime


class ShipmentTimelineInfos(NamedTuple):
    shipment_id: str
    created_at: date
    total_transit_time: int
    arrival_date: date or None
    delivery_date: date or None


def total_tasks_weight(tasks_df: pd.DataFrame) -> pd.DataFrame:
    tasks_df = tasks_df.loc[:, ["shipment_id", "weight"]].groupby("shipment_id").sum()
    return tasks_df.rename(columns={"weight": "total_tasks_weight"})


def daily_tasks_total_weight_on_shipment(tasks_df: pd.DataFrame, date: date) -> int:
    is_todo_task_due_date = (tasks_df["status"] == "to_do") & (tasks_df["due_date"] == date)
    is_done_task_done_date = (tasks_df["status"] == "done") & (tasks_df["done_date"] == date)
    tasks_todo_today_on_shipment_df = tasks_df[(is_todo_task_due_date | is_done_task_done_date)]
    if len(tasks_todo_today_on_shipment_df["shipment_id"]) == 0:
        return 0
    daily_tasks_todo_total_weight_df = (
        tasks_todo_today_on_shipment_df.loc[:, ["shipment_id", "weight"]].groupby("shipment_id").sum()
    )
    return daily_tasks_todo_total_weight_df.weight.iat[0]


def shipment_total_workflows_weight(active_shipments_data_df: pd.DataFrame,) -> pd.DataFrame:
    active_shipments_data_df = active_shipments_data_df.groupby("shipment_id", as_index=False).agg(
        {
            "total_workflow_weight": sum,
            **{
                col: "first"
                for col in active_shipments_data_df.columns
                if col not in ["shipment_id", "total_workflow_weight"]
            },
        }
    )
    active_shipments_data_df = active_shipments_data_df.rename(
        columns={"total_workflow_weight": "total_workflows_weight"}
    )
    return active_shipments_data_df


def shipments_remaining_days(
    active_shipments_data_df: pd.DataFrame, today: date, shipments_timelines_df: pd.DataFrame
) -> pd.DataFrame:
    total_remaining_days = []
    for shipment_id in active_shipments_data_df["shipment_id"]:
        shipment_timeline = shipments_timelines_df.loc[shipments_timelines_df["shipment_id"] == shipment_id, :]
        shipment_future_days = shipment_timeline.loc[shipment_timeline["dates"] > today, :]
        total_remaining_days.append(len(shipment_future_days))
    active_shipments_data_df.loc[:, "remaining_days"] = total_remaining_days
    return active_shipments_data_df


def get_shipment_start_date(shipment_tup: ShipmentTimelineInfos) -> date:
    return shipment_tup.created_at


def get_shipment_end_date(shipment_tup: ShipmentTimelineInfos) -> date or None:
    if not pd.isnull(shipment_tup.delivery_date):
        return shipment_tup.delivery_date
    if not pd.isnull(shipment_tup.arrival_date):
        return shipment_tup.arrival_date
    return None


def shipment_date_range(shipment_tup: ShipmentTimelineInfos) -> pd.DatetimeIndex:
    start_date = get_shipment_start_date(shipment_tup)
    end_date = get_shipment_end_date(shipment_tup)
    if pd.isnull(end_date):
        transit_time = shipment_tup.total_transit_time
        return pd.date_range(start=start_date, periods=transit_time)
    return pd.date_range(start=start_date, end=(end_date + timedelta(days=1)))


def shipments_timelines(active_shipments_data_df: pd.DataFrame) -> pd.DataFrame:
    shipments_timeline_infos_grouped = (
        active_shipments_data_df.loc[
            :, ["shipment_id", "created_at", "total_transit_time", "arrival_date", "delivery_date"]
        ]
        .groupby(["shipment_id", "total_transit_time"], as_index=False)
        .agg({col: "first" for col in ["created_at", "delivery_date", "arrival_date"]})
    )
    shipments_timelines_list = []
    for tup in shipments_timeline_infos_grouped.itertuples(True, "ShipmentTimelineInfos"):
        print(tup)
        shipment_timeline_df = pd.DataFrame(data={"shipment_id": [], "dates": []})
        dates = shipment_date_range(shipment_tup=tup)
        shipment_timeline_df["dates"] = dates
        shipment_timeline_df["shipment_id"] = tup.shipment_id
        shipments_timelines_list.append(shipment_timeline_df)
    shipments_timelines_df = pd.concat(shipments_timelines_list)
    shipments_timelines_df.loc[:, "dates"] = shipments_timelines_df["dates"].dt.date
    return shipments_timelines_df.reset_index(drop=True)


def estimated_remaining_daily_workload(
    active_shipments_data_df: pd.DataFrame, tasks_df: pd.DataFrame, today: date, shipments_timelines_df: pd.DataFrame,
) -> pd.DataFrame:
    active_shipments_data_df = active_shipments_data_df[
        [
            "shipment_id",
            "shipment_status",
            "freight_method",
            "freight_type",
            "total_workflow_weight",
            "foresea_name",
            "incoterm",
            "total_transit_time",
            "created_at",
            "delivery_date",
            "arrival_date",
        ]
    ]
    active_shipments_data_df = shipment_total_workflows_weight(active_shipments_data_df=active_shipments_data_df,)
    active_shipments_data_df = shipments_remaining_days(active_shipments_data_df, today, shipments_timelines_df)
    total_tasks_weight_df = total_tasks_weight(tasks_df=tasks_df)
    active_shipments_data_df = pd.merge(active_shipments_data_df, total_tasks_weight_df, on="shipment_id", how="left")
    active_shipments_data_df.loc[:, "estimated_remaining_daily_workload"] = (
        active_shipments_data_df["total_workflows_weight"] - active_shipments_data_df["total_tasks_weight"]
    ) / active_shipments_data_df["remaining_days"]
    active_shipments_data_df.loc[
        ~np.isfinite(active_shipments_data_df["estimated_remaining_daily_workload"]),
        "estimated_remaining_daily_workload",
    ] = 0
    return active_shipments_data_df


def daily_workload_calculation(shipments_timelines_df: pd.DataFrame, tasks_df: pd.DataFrame) -> pd.DataFrame:
    tasks_df.loc[:, "due_date"] = tasks_df["due_date"].dt.date
    tasks_df.loc[:, "done_date"] = tasks_df["done_date"].dt.date
    shipments_timelines_df.loc[:, "created_tasks_weight"] = shipments_timelines_df.apply(
        lambda row: daily_tasks_total_weight_on_shipment(
            tasks_df=tasks_df.loc[tasks_df["shipment_id"] == row["shipment_id"], :], date=row["dates"],
        ),
        axis=1,
    )
    shipments_timelines_df.loc[:, "daily_workload"] = (
        shipments_timelines_df["created_tasks_weight"] + shipments_timelines_df["estimated_remaining_daily_workload"]
    )
    return shipments_timelines_df


def shipments_daily_workload_timelines(
    active_shipments_data_df: pd.DataFrame, tasks_df: pd.DataFrame, today: date,
) -> pd.DataFrame:
    tasks_df.loc[:, "weight"] = tasks_df["weight"].fillna(0)
    tasks_df = tasks_df.loc[:, ["shipment_id", "due_date", "status", "weight", "updated_at", "done_date"]]
    shipments_timelines_df = shipments_timelines(active_shipments_data_df)
    active_shipments_data_df = estimated_remaining_daily_workload(
        active_shipments_data_df=active_shipments_data_df,
        tasks_df=tasks_df,
        today=today,
        shipments_timelines_df=shipments_timelines_df,
    )
    shipments_timelines_df = pd.merge(shipments_timelines_df, active_shipments_data_df, how="left", on="shipment_id")
    shipments_timelines_df.loc[shipments_timelines_df["dates"] <= today, "estimated_remaining_daily_workload"] = 0.0
    shipments_timelines_df = daily_workload_calculation(
        shipments_timelines_df=shipments_timelines_df, tasks_df=tasks_df
    )
    return shipments_timelines_df[["shipment_id", "dates", "daily_workload"]]



import pandas as pd


def pivot_category_prices(prices_df):
    prices_df = pd.pivot_table(
        prices_df,
        index=["shipment_id"],
        columns=["category"],
        values=["price_in_eur", "initial_price_in_eur", "purchase_price_in_eur", "initial_purchase_price_in_eur",],
    )
    prices_df.columns = ["_".join(col).strip() for col in prices_df.columns.values]
    prices_df = prices_df.rename(
        columns={
            "price_in_eur_arrival_fees": "arrival_fees_price",
            "price_in_eur_arrival_truck_freight": "arrival_truck_freight_price",
            "price_in_eur_customs": "customs_price",
            "price_in_eur_departure_fees": "departure_fees_price",
            "price_in_eur_departure_truck_freight": "departure_truck_freight_price",
            "price_in_eur_freight": "freight_price",
            "price_in_eur_insurance": "insurance_price",
            "price_in_eur_other": "other_price",
            "initial_price_in_eur_arrival_fees": "arrival_fees_initial_price",
            "initial_price_in_eur_arrival_truck_freight": "arrival_truck_freight_initial_price",
            "initial_price_in_eur_departure_fees": "departure_fees_initial_price",
            "initial_price_in_eur_departure_truck_freight": "departure_truck_freight_initial_price",
            "initial_price_in_eur_freight": "freight_initial_price",
            "initial_price_in_eur_insurance": "insurance_initial_price",
            "purchase_price_in_eur_arrival_fees": "arrival_fees_purchase_price",
            "purchase_price_in_eur_arrival_truck_freight": "arrival_truck_freight_purchase_price",
            "purchase_price_in_eur_departure_fees": "departure_fees_purchase_price",
            "purchase_price_in_eur_departure_truck_freight": "departure_truck_freight_purchase_price",
            "purchase_price_in_eur_freight": "freight_purchase_price",
            "purchase_price_in_eur_insurance": "insurance_purchase_price",
            "initial_purchase_price_in_eur_arrival_fees": "arrival_fees_initial_purchase_price",
            "initial_purchase_price_in_eur_arrival_truck_freight": "arrival_truck_freight_initial_purchase_price",
            "initial_purchase_price_in_eur_departure_fees": "departure_fees_initial_purchase_price",
            "initial_purchase_price_in_eur_departure_truck_freight": "departure_truck_freight_initial_purchase_price",
            "initial_purchase_price_in_eur_freight": "freight_initial_purchase_price",
            "initial_purchase_price_in_eur_insurance": "insurance_initial_purchase_price",
        }
    )
    return prices_df.drop(
        [
            "initial_price_in_eur_customs",
            "initial_price_in_eur_other",
            "purchase_price_in_eur_customs",
            "purchase_price_in_eur_other",
        ],
        axis=1,
    )
